﻿using UnityEngine;
using System.Collections;

public class AudioController : MonoBehaviour
{
    [SerializeField]
    private AudioClip menuMusic;
    
    [SerializeField]
    private AudioClip levelMusic;

    [SerializeField]
    private AudioSource source;
    
    static private AudioController instance;
    static private bool soundsFX;
    static private bool BGMmusic;

    protected virtual void Awake()
    {
        // Singleton enforcement
        if (instance == null)
        {
            // Register as singleton if first
            instance = this;
            DontDestroyOnLoad(this);
        }
        else
        {
            // Self-destruct if another instance exists
            Destroy(this);
            return;
        }
    }

    protected virtual void Start()
    {
        // If the game starts in a menu scene, play the appropriate music
        PlayMenuMusic();
        soundsFX = true;
        BGMmusic = true;
    }
    
    public static void toogleMusic(bool state)
    {
        if (instance!=null)
        {
            if (instance != null)
            {
                if(state)
                    instance.source.Play();
                else
                    instance.source.Stop();

                BGMmusic = state;
            }
        }
    }

    public static bool isFXActive()
    {
        return soundsFX;
    }

    public static bool isBGMActive()
    {
        return BGMmusic;
    }

    public static void toogleSoundFX(bool state)
    {
        if (instance != null)
        {
            if (instance != null)
            {
                soundsFX = state;
            }
        }
    }

    public static void ChangeVolume(float amount)
    {
        instance.source.volume = amount;
    }

    static public void PlayMenuMusic()
    {
        if (instance != null)
        {
            if (instance.source != null)
            {
                instance.source.Stop();
                instance.source.clip = instance.menuMusic;
                instance.source.Play();
                BGMmusic = true;
            }
        }
        else
        {
            Debug.LogError("Unavailable MusicPlayer component");
        }
    }
    
    static public void PlayGameMusic()
    {
        if (instance != null)
        {
            if (instance.source != null)
            {
                instance.source.Stop();
                instance.source.clip = instance.levelMusic;
                instance.source.Play();
            }
        }
        else
        {
            Debug.LogError("Unavailable MusicPlayer component");
        }
    }
}
