﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityStandardAssets.CrossPlatformInput;

public class GoblinControllerScript : MonoBehaviour {

    public float maxSpeed = 10f;
    public int simultBullets = 1;
    public int speedOfBullets = 5;
    public float maxFallTime = 3;
    public float CurrentFallTime;
    bool facingRight = true;
    bool PlayerIsFalling;
    Animator anim;

    //Android controls
    public float moveForce;

    // Check grounding
    bool grounded = true;
    public Transform groundCheck;
    public float groundRadius = 0.1f;
    public LayerMask whatIsGround;

    // Jumping and falling
    public float jumpForce = 5f;
    public float fallMultiplier = 2.5f;
    public float lowJumpMultiplier = 2f;

    public Transform shootPrefab;
    private int numberOfBalls = 1;
    private int maxFireballs = 4;
    private Rigidbody2D rigidComp;

    //Boosts
    public int boostSeconds = 8;
    public float randomInvul = 0.5f;
    public float randomShoots = 0.5f;
    public float randomHeal = 0.5f;

    private Vector2 lastCheckpointPos;   //lastCheckpoint Position
    private DamageableItem dmgItem;
    private AudioSource audio;
    public AudioClip jumpClip;
    private Camera cam;

    public int secondsToWaitUntilDmgAgain;

    void Awake () {
        anim = GetComponent<Animator>();
        dmgItem = this.GetComponent<DamageableItem>();
        audio = GetComponent<AudioSource>();
        rigidComp = GetComponent<Rigidbody2D>();
        cam = GameObject.FindWithTag("MainCamera").GetComponent<Camera>();
    }

    private void Start()
    {
        if (PlayerPrefs.GetInt("maxFireballs") > maxFireballs)
            maxFireballs = PlayerPrefs.GetInt("maxFireballs");
        SaveLastCheckpointPos();
    }


    private void Update()
    {
        float characterMoving;
        bool attacking;
        Vector2 direction = new Vector2(0, 0);

        //Android input
        float moveAndroid;
        bool isJumpingAndroid = false;
        bool attackAndroid = false;

        //Default input
        bool isJumping = Input.GetButtonDown("Jump");
        float move = Input.GetAxis("Horizontal");
        bool attack = Input.GetMouseButtonDown(0); //Left click

        if (ApplicationUtil.platform == RuntimePlatform.Android)
        {
            moveAndroid = CrossPlatformInputManager.GetAxis("Horizontal");
            isJumpingAndroid = CrossPlatformInputManager.GetButton("Jump");
            if (Input.touchCount > 0)
            {
                attackAndroid = (Input.GetTouch(0).phase == TouchPhase.Began);
            }
            characterMoving = moveAndroid;
            attacking = attackAndroid;
        }
        else
        {
            characterMoving = move;
            attacking = attack;
        }

        //Check grounding
        grounded = Physics2D.OverlapCircle(groundCheck.position, groundRadius, whatIsGround);

        if (grounded && (isJumping || isJumpingAndroid))
        {
            //anim.SetBool("Ground", false);
            if(!PlayerIsFalling)
            {
                rigidComp.angularVelocity = 0f;
                rigidComp.velocity = Vector2.zero;
                rigidComp.AddForce(new Vector2(0, jumpForce));
                if (AudioController.isFXActive())
                {
                    audio.clip = jumpClip;
                    audio.Play();
                }
            }
        }

        //Check falling
        if (!grounded)
        {
            PlayerIsFalling = true;
        }
        else PlayerIsFalling = false;

        if (PlayerIsFalling)
        {
            CurrentFallTime += Time.deltaTime;
        }
        else
            CurrentFallTime = 0;

        //Basic movement of the character
        rigidComp.velocity = new Vector2(characterMoving * maxSpeed, rigidComp.velocity.y);


        //Attacking
        anim.SetBool("Attack", attacking);
        if (attacking)
        {
            Vector2 target = Camera.main.ScreenToWorldPoint(new Vector2(Input.mousePosition.x, Input.mousePosition.y));
            Vector2 myPos = new Vector2(transform.position.x, transform.position.y);
            direction = target - myPos;
            direction.Normalize();
            Quaternion rotation = Quaternion.Euler(0, 0, Mathf.Atan2(direction.y, direction.x) * Mathf.Rad2Deg);

            for (int i = 0; i < numberOfBalls; i++)
            {
                Transform projectile = Instantiate(
                    shootPrefab,
                    myPos,
                    rotation);
                projectile.GetComponent<Collider2D>().isTrigger = false;
                projectile.GetComponent<Rigidbody2D>().velocity = direction * speedOfBullets;
            }

        }

        //Flipping when moving or shooting in a direction x
        if (((characterMoving > 0) || (direction.x > 0)) && !facingRight)
            Flip();
        else if (((characterMoving < 0) || (direction.x < 0)) && facingRight)
            Flip();

        //Animation parameters
        anim.SetFloat("Speed", Mathf.Abs(characterMoving));
        anim.SetBool("Ground", grounded);

    }

    void Flip()
    {
        facingRight = !facingRight;
        Vector3 theScale = transform.localScale;
        theScale.x *= -1;
        transform.localScale = theScale;
    }

    public void SaveLastCheckpointPos()
    {
        lastCheckpointPos = transform.position;
    }

    public void RestoreLastCheckpointPos()
    {
        rigidComp.angularVelocity = 0f;
        rigidComp.velocity = Vector2.zero;
        transform.position = lastCheckpointPos;
        cam.GetComponent<UnityStandardAssets._2D.Camera2DFollow>().damping = 0;
        cam.transform.position = lastCheckpointPos;
        cam.GetComponent<UnityStandardAssets._2D.Camera2DFollow>().damping = 1;
    }


    public IEnumerator AddBoost()
    {
        if (UnityEngine.Random.value < randomShoots)
        {
            numberOfBalls = UnityEngine.Random.Range(2, maxFireballs);
            yield return new WaitForSeconds(boostSeconds);
            numberOfBalls = 1;
        }
        else if (UnityEngine.Random.value < randomHeal)
        {
            dmgItem.RandomHeal();
        }
        else if (UnityEngine.Random.value < randomInvul)
        {
            Invulnerability.TriggerInvulnerability();
        }
        else
        {
            dmgItem.Heal(10f);
        }

        StopCoroutine(AddBoost());
    }
}
