﻿using System;
using System.Collections;
using UnityEngine;

public class CollisionItem : MonoBehaviour {
    public int damage = 10;                     //Damage this item will do
    public bool continuousDamage = false;        //ContinuousDamage while colliders touch
    public int secondsToWaitUntilDmgAgain = 3;  //Seconds to wait with invulnerability
    private DamageableItem dmgAbleItem;         //Item that takes the damage
    private Collider2D collider;
    public LayerMask destroyTouchingWith;       //Limits of fireballs

    private void Awake()
    {
        collider = GetComponent<Collider2D>();

        if (this.gameObject.name.Contains("FireBallShot"))
        {
            if(PlayerPrefs.GetFloat("fireballDamage") != 0)
                damage = Mathf.RoundToInt(damage * PlayerPrefs.GetFloat("fireballDamage"));
        }

        if ((LayerMask.LayerToName(this.gameObject.layer).Equals("Harming")) || (LayerMask.LayerToName(this.gameObject.layer).Equals("Enemies")))
            if(PlayerPrefs.GetFloat("enemyDamageProportion") > 0)
                damage = Mathf.RoundToInt(damage * PlayerPrefs.GetFloat("enemyDamageProportion"));

    }

    void OnCollisionEnter2D(Collision2D coll)
    {
        Collisioned(coll.collider);
        CollisionedFireBall(coll.collider);
    }

    private void OnTriggerEnter2D(Collider2D collision)
    {
        Collisioned(collision);
        CollisionedFireBall(collision);
    }

    void Collisioned(Collider2D coll)
    {
        if ((LayerMask.LayerToName(this.gameObject.layer).Equals("Harming")) || (LayerMask.LayerToName(this.gameObject.layer).Equals("Enemies")) || (LayerMask.LayerToName(this.gameObject.layer).Equals("Traps")))  //All the objects that do damage to 
        {
            if (coll.gameObject.CompareTag("Player")) //the player
            {
                dmgAbleItem = coll.gameObject.GetComponent<DamageableItem>();

                if (!dmgAbleItem.Invulnerability)
                    dmgAbleItem.TakeDamage(damage);

                if (!continuousDamage && !dmgAbleItem.Invulnerability)
                {
                    if (this.gameObject.CompareTag("Shot"))
                    {
                        this.GetComponentInParent<Renderer>().enabled = false;
                        collider.enabled = false;
                    }
                    Invulnerability.TriggerInvulnerability();
                }

            }
            else if (collider.IsTouchingLayers(destroyTouchingWith) && this.gameObject.CompareTag("Shot"))
            {
                Destroy(this.gameObject);
            }
        }        
    }

    private void CollisionedFireBall(Collider2D coll)
    {
        if (this.gameObject.name.Contains("FireBallShot"))  //Bullet of Character does damage to the enemies
        {
            if (coll.gameObject.CompareTag("Enemy"))
            {
                dmgAbleItem = coll.gameObject.GetComponent<DamageableItem>();
                dmgAbleItem.TakeDamage(damage);
                if(coll.IsTouchingLayers(destroyTouchingWith))
                    Destroy(this.gameObject);
            }
            else if (!coll.gameObject.CompareTag("Player") && !coll.gameObject.name.Contains("FireBallShot"))
            {
                Destroy(this.gameObject);
            }
        }
    }
}
