﻿using UnityEngine;
using System;

[System.Serializable]
public class MapGeneration : MonoBehaviour
{

    public GameObject groundTop, bridge, spikes, groundMid, flyingGround, lava, boost, crystal, checkPoint, summoner, wizard, sorcerer, door;
    private GameObject lastGroundTop;
    public int minPlatformSize = 1;
    public int maxPlatformSize = 10;
    public int finalBossPlatform = 16;
    public int numberGroundMid = 8;
    public int maxHazardSize = 3;
    public int maxHeight = 3;
    public int maxDrop = -2;
    public float numberOfCheckpoints = 3;
    public int platforms = 100;
    [Range(0.0f, 1f)]
    public float hazardChance = .5f;
    [Range(0.0f, 1f)]
    public float bridgeChance = .25f;
    [Range(0.0f, 1f)]
    public float spikeChance = .5f;
    [Range(0.0f, 1f)]
    public float flyingGroundChance = .1f;
    [Range(0.0f, 1f)]
    public float lavaChance = .5f;
    [Range(0.0f, 1f)]
    public float crystalChance = .1f;
    [Range(0.0f, 1f)]
    public float boostChance = .1f;
    [Range(0.0f, 1f)]
    public float summonerChance = .4f;
    [Range(0.0f, 1f)]
    public float sorcererChance = .5f;

    private int blockNum = 1;
    private int blockHeight;
    private bool isHazard;
    private float checkPointIndex;


    private void LoadPreferences()
    {
        platforms = PlayerPrefs.GetInt("platforms");
        hazardChance = PlayerPrefs.GetFloat("hazardChance");
        bridgeChance = PlayerPrefs.GetFloat("bridgeChance");
        spikeChance = PlayerPrefs.GetFloat("spikeChance");
        flyingGroundChance = PlayerPrefs.GetFloat("flyingGroundChance");
        lavaChance = PlayerPrefs.GetFloat("lavaChance");
        crystalChance = PlayerPrefs.GetFloat("crystalChance");
        boostChance = PlayerPrefs.GetFloat("boostChance");
        summonerChance = PlayerPrefs.GetFloat("summonerChance");
        sorcererChance = PlayerPrefs.GetFloat("sorcererChance");
    }

    private void SaveInitialPreferences()
    {
        PlayerPrefs.SetInt("platforms", platforms);
        PlayerPrefs.SetFloat("hazardChance", hazardChance);
        PlayerPrefs.SetFloat("bridgeChance", bridgeChance);
        PlayerPrefs.SetFloat("spikeChance", spikeChance);
        PlayerPrefs.SetFloat("flyingGroundChance", flyingGroundChance);
        PlayerPrefs.SetFloat("lavaChance", lavaChance);
        PlayerPrefs.SetFloat("crystalChance", crystalChance);
        PlayerPrefs.SetFloat("boostChance", boostChance);
        PlayerPrefs.SetFloat("summonerChance", summonerChance);
        PlayerPrefs.SetFloat("sorcererChance", sorcererChance);
        PlayerPrefs.SetInt("level", 1);
    }

    private void IncreaseDifficult(int level)
    {
        PlayerPrefs.SetInt("platforms", platforms + (level));
        PlayerPrefs.SetFloat("hazardChance", hazardChance + (level / 20));
        PlayerPrefs.SetFloat("bridgeChance", bridgeChance + (level / 20));
        PlayerPrefs.SetFloat("spikeChance", spikeChance + (level / 20));
        PlayerPrefs.SetFloat("flyingGroundChance", flyingGroundChance + (level / 20));
        PlayerPrefs.SetFloat("lavaChance", lavaChance + (level / 20));
        PlayerPrefs.SetFloat("crystalChance", crystalChance - (level / 20));
        PlayerPrefs.SetFloat("boostChance", boostChance - (level / 20));
        PlayerPrefs.SetFloat("summonerChance", summonerChance + (level / 20));
        PlayerPrefs.SetFloat("sorcererChance", sorcererChance + (level / 20));
        if (PlayerPrefs.GetFloat("enemyDamageProportion") <= 0)
        {
            PlayerPrefs.SetFloat("enemyDamageProportion", 1);
        }
        else
        {
            PlayerPrefs.SetFloat("enemyDamageProportion", PlayerPrefs.GetFloat("enemyDamageProportion") * 1.2f);
        }

        if (PlayerPrefs.GetFloat("enemyHealthProportion") <= 0)
        {
            PlayerPrefs.SetFloat("enemyHealthProportion", 1);
        }
        else
        {
            PlayerPrefs.SetFloat("enemyHealthProportion", PlayerPrefs.GetFloat("enemyHealthProportion") * 1.2f);
        }

        if (PlayerPrefs.GetFloat("fireballDamage") <= 0)
        {
            PlayerPrefs.SetFloat("fireballDamage", 1);
        }
    }

    // Use this for initialization
    void Start()
    {

        if (PlayerPrefs.GetInt("level") > 1)
        {
            IncreaseDifficult(PlayerPrefs.GetInt("level"));
            LoadPreferences();
        }
        else if(PlayerPrefs.GetInt("level") <= 0)
        {
            SaveInitialPreferences();
        }


        BoxCollider2D groundTopCol = groundTop.GetComponent<BoxCollider2D>();

        //begining tile
        Instantiate(groundTop, new Vector2(0, 0), Quaternion.identity);
        for (int grdMid = 1; grdMid < numberGroundMid; grdMid++)
        {
            Instantiate(groundMid, new Vector2(0, (blockHeight - grdMid) * groundTopCol.size.y), Quaternion.identity);
        }

        //First checkpoint calculation and initialitzation of array
        checkPointIndex = 1;

        //Number of total plarforms
        for (int ptfm = 1; ptfm < platforms; ptfm++)
        {

            //Randomizing hazards
            if (isHazard)
            {
                isHazard = false;
            }
            else
            {
                if (UnityEngine.Random.value < hazardChance)
                {
                    isHazard = true;
                }
                else
                {
                    isHazard = false;
                }
            }

            if (isHazard)
            {
                int hazardSize = Mathf.RoundToInt(UnityEngine.Random.Range(1, maxHazardSize));

                //Randomizing spikes
                if (UnityEngine.Random.value < spikeChance)
                {
                    //generate spike trap
                    for (int tiles = 0; tiles < hazardSize; tiles++)
                    {
                        Instantiate(spikes, new Vector2(blockNum * groundTopCol.size.x, (blockHeight) * groundTopCol.size.y), Quaternion.identity);
                        for (int grdMid = 1; grdMid < numberGroundMid; grdMid++)
                        {
                            Instantiate(groundMid, new Vector2((blockNum * groundTopCol.size.x), (blockHeight - grdMid) * groundTopCol.size.y), Quaternion.identity);
                        }

                        blockNum++;
                    }
                }
                else
                {
                    //Randomizing holes and lava
                    if (UnityEngine.Random.value < lavaChance)
                    {
                        Instantiate(lava, new Vector2((blockNum * groundTopCol.size.x) - 0.1f, (blockHeight) * groundTopCol.size.y), Quaternion.identity);
                    }
                    blockNum += hazardSize;
                }
            }
            else
            {
                int platformSize = UnityEngine.Random.Range(minPlatformSize, maxPlatformSize);
                blockHeight = blockHeight + UnityEngine.Random.Range(maxDrop, maxHeight);

                if (UnityEngine.Random.value < bridgeChance)
                {
                    for (int tiles = 0; tiles < platformSize; tiles++)
                    {
                        if (tiles == 0 || tiles == platformSize - 1)
                        {
                            lastGroundTop = Instantiate(groundTop, new Vector2(blockNum * groundTopCol.size.x, blockHeight * groundTopCol.size.y), Quaternion.identity);

                            for (int grdMid = 1; grdMid < numberGroundMid; grdMid++)
                            {
                                Instantiate(groundMid, new Vector2(blockNum * groundTopCol.size.x, (blockHeight - grdMid) * groundTopCol.size.y), Quaternion.identity);
                            }
                        }
                        else
                        {
                            Instantiate(bridge, new Vector2(blockNum * groundTopCol.size.x, blockHeight * groundTopCol.size.y), Quaternion.identity);
                        }
                        blockNum++;
                    }
                }
                else
                {
                    //Default ground generation
                    for (int tiles = 0; tiles < platformSize; tiles++)
                    {
                        lastGroundTop = Instantiate(groundTop, new Vector2(blockNum * groundTopCol.size.x, blockHeight * groundTopCol.size.y), Quaternion.identity);
                        for (int grdMid = 1; grdMid < numberGroundMid; grdMid++)
                        {
                            Instantiate(groundMid, new Vector2(blockNum * groundTopCol.size.x, (blockHeight - grdMid) * groundTopCol.size.y), Quaternion.identity);
                        }

                        //Flying grounds with boosts, crystals or enemies
                        if (UnityEngine.Random.value < flyingGroundChance)
                        {
                            Instantiate(flyingGround, new Vector2(blockNum * groundTopCol.size.x, (blockHeight + 2) * groundTopCol.size.y), Quaternion.identity);
                            if (UnityEngine.Random.value < crystalChance)
                            {
                                Instantiate(crystal, new Vector2(blockNum * groundTopCol.size.x, (blockHeight + 3.5f) * groundTopCol.size.y), Quaternion.identity);
                            }
                            else if (UnityEngine.Random.value < boostChance)
                            {
                                Instantiate(boost, new Vector2(blockNum * groundTopCol.size.x, (blockHeight + 3.5f) * groundTopCol.size.y), Quaternion.identity);
                            }
                            else if (UnityEngine.Random.value < summonerChance)
                            {
                                Instantiate(summoner, new Vector2(blockNum * groundTopCol.size.x, (blockHeight + 3.5f) * groundTopCol.size.y), Quaternion.identity);
                            }
                            else if (UnityEngine.Random.value < sorcererChance)
                            {
                                Instantiate(sorcerer, new Vector2(blockNum * groundTopCol.size.x, (blockHeight + 3.5f) * groundTopCol.size.y), Quaternion.identity);
                            }

                        }
                        blockNum++;
                    }
                }
            }

            //Controlling checkpoint generation
            if (checkPointIndex <= numberOfCheckpoints)
            {
                float proportion = checkPointIndex / (numberOfCheckpoints + 1); //We add one more for having the correct number of checkpoints, because it depends in the proportion of the map
                if ((Mathf.RoundToInt(proportion * platforms)) == ptfm)
                {
                    GameObject cp = Instantiate(checkPoint, new Vector2(lastGroundTop.transform.position.x, lastGroundTop.transform.position.y), Quaternion.identity);
                    cp.transform.position = lastGroundTop.transform.position;
                    checkPointIndex++;
                }
            }
        }

        //When all platforms are created, we will do the final platform with the final boss and a checkpoint
        GameObject finalCheck = Instantiate(checkPoint, new Vector2(lastGroundTop.transform.position.x, lastGroundTop.transform.position.y), Quaternion.identity);
        finalCheck.transform.position = lastGroundTop.transform.position;
        for (int ptfm = 1; ptfm < finalBossPlatform; ptfm++)
        {
            lastGroundTop = Instantiate(groundTop, new Vector2(blockNum * groundTopCol.size.x, blockHeight * groundTopCol.size.y), Quaternion.identity);

            for (int grdMid = 1; grdMid < numberGroundMid; grdMid++)
            {
                Instantiate(groundMid, new Vector2(blockNum * groundTopCol.size.x, (blockHeight - grdMid) * groundTopCol.size.y), Quaternion.identity);
            }

            if (ptfm == (finalBossPlatform / 2))
            {
                Instantiate(wizard, new Vector2(lastGroundTop.transform.position.x, lastGroundTop.transform.position.y+1), Quaternion.identity);
            }

            blockNum++;
        }
        Instantiate(door, new Vector2(lastGroundTop.transform.position.x, lastGroundTop.transform.position.y + 0.75f), Quaternion.identity);
        if (AstarPath.active.isActiveAndEnabled)
            AstarPath.active.Scan();
    }
}
