﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.SceneManagement;
using UnityEngine.UI;

public class SceneLoader : MonoBehaviour {

    private bool paused;
    public GameObject pauseMenuUI, finishMenuUI, shopMenuUI, levelUI, gameOverUI, retryUI;
    public static SceneLoader instance;
    private static Text[] texts;
    public static int level;

    private void Awake()
    {
        // Singleton enforcement
        if (instance == null)
        {
            // Register as singleton if first
            instance = this;
            if (PlayerPrefs.GetInt("level") <= 0)
            {
                PlayerPrefs.SetInt("level", 1);
                level = 1;
                levelUI.transform.GetChild(1).GetComponent<Text>().text = "1";
            }
            else
            {
                level = PlayerPrefs.GetInt("level");
                PlayerPrefs.SetInt("level", level);
                levelUI.transform.GetChild(1).GetComponent<Text>().text = level.ToString();
            }
            DontDestroyOnLoad(this);
        }
        else
        {
            // Self-destruct if another instance exists
            Destroy(this);
            return;
        }
    }

    private void Update()
    {
        if(Input.GetKeyDown(KeyCode.Escape))
        {
            if (!paused)
                PauseMenu(true);
            else
                PauseMenu(false);
        }
    }
    
    public void StartScene()
    {
        level = 1;
        Time.timeScale = 1;
        levelUI.transform.GetChild(1).GetComponent<Text>().text = level.ToString();
        PlayerPrefs.DeleteAll();
        SceneManager.LoadScene(2);
    }

    public void LoadNextScene()
    {
        Time.timeScale = 1;
        if (PlayerPrefs.GetInt("level") != 0)
        {
            levelUI.transform.GetChild(1).GetComponent<Text>().text = level.ToString();
            SceneManager.LoadScene(2, LoadSceneMode.Single);
        }
        else
            StartScene();
    }

    public void LoadScene(int scene)
    {
        Time.timeScale = 1;
        SceneManager.LoadScene(scene);
    }
    
    public void PauseMenu(bool pause)
    {
        paused = pause;
        pauseMenuUI.SetActive(pause);
        if (pause)
        {
            Time.timeScale = 0;
        }
        else
        {
            Time.timeScale = 1;
        }
    }
   
    public void Finish()
    {
        if(finishMenuUI!=null)
        {
            finishMenuUI.SetActive(true);
            texts = finishMenuUI.GetComponentsInChildren<Text>();
            for (int i = 0; i < texts.Length; i++)
            {
                if (texts[i].tag.Equals("Score"))
                {
                    texts[i].text = ScoreManager.getcPoints().ToString();
                }
                else if (texts[i].tag.Equals("Coins"))
                {
                    texts[i].text = ScoreManager.getcCrystals().ToString();
                }
            }
            PlayerPrefs.SetInt("saved", 1);
            level += 1;
            PlayerPrefs.SetInt("level", level);
            Time.timeScale = 0;
        }
    }

    public static void FinishMenu()
    {
        if(instance!=null)
            instance.Finish();
    }

    public static void LoadGameOverMenu()
    {
        if (instance != null)
            instance.LoadGameOver();
    }

    public void LoadGameOver()
    {
        Time.timeScale = 0;
        PlayerPrefs.DeleteAll();
        level = 1;
        PlayerPrefs.SetInt("level", level);
        gameOverUI.SetActive(true);
    }

    public static void LoadRetryMenu()
    {
        if (instance != null)
            instance.LoadRetry();
    }

    public void LoadRetry()
    {
        Time.timeScale = 0;
        retryUI.SetActive(true);
    }

    public void Retry()
    {
        retryUI.SetActive(false);
        Time.timeScale = 1;
        CheckpointManager.RestoreToLastCheckpoint();
    }

    public static void StopTime()
    {
        Time.timeScale = 0;
    }

    public static void ResumeTime()
    {
        Time.timeScale = 1;
    }

    public void LoadShop()
    {
        shopMenuUI.SetActive(true);
    }

    public void ExitShop()
    {
        shopMenuUI.SetActive(false);
    }

    public void QuitGame()
    {
        Application.Quit();
    }
}
