﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Events;
using UnityEngine.UI;

public class Shop : MonoBehaviour {

    public Transform [] slots;
    public Transform buyCristalButton;
    public Sprite fireBallSprite, boostSprite;
    private Transform[] buttons, images, texts;

	// Use this for initialization
	void Start () {

        buttons = new Transform[slots.Length];
        images = new Transform[slots.Length];
        texts = new Transform[slots.Length];

        DontDestroyOnLoad(this);

        if (slots!=null)
        {
            for(int i = 0; i<slots.Length;i++)
            {
                int index = i;
                images[i] = slots[i].GetChild(0);
                buttons[i] = slots[i].GetChild(1);
                texts[i] = slots[i].GetChild(2);
                buttons[i].GetComponent<MultiImageButton>().onClick.AddListener(() => Unlock(index));

                if (PlayerPrefs.GetInt("unlockedSkill" + index) == 1) //UnlockedSkill
                {
                    AlreadyUnlocked(index);
                    Unlockable(index + 1);
                }
            }
        }

        if(buyCristalButton!=null)
        {
            buyCristalButton.GetComponent<MultiImageButton>().onClick.AddListener(() => ScoreManager.UsecPoints(100));
        }
	}

    public void AlreadyUnlocked(int index)
    {
        Destroy(buttons[index].gameObject);
        images[index].GetComponent<Image>().sprite = GetSprite(index);
        texts[index].gameObject.GetComponent<Text>().text = GetText(index);
        texts[index].gameObject.SetActive(true);
    }

    public void Unlock(int index)
    {
        int cristals = ScoreManager.getcCrystals();
        int cost = int.Parse(slots[index].GetComponentInChildren<Text>().text);
        if (cristals >= cost)
        {
            ScoreManager.UsecCrystals(cost);
            Destroy(buttons[index].gameObject);
            images[index].GetComponent<Image>().sprite = GetSprite(index);
            texts[index].gameObject.GetComponent<Text>().text = GetText(index);
            texts[index].gameObject.SetActive(true);
            PlayerPrefs.SetInt("unlockedSkill" + index, 1);

            int unlockable = PlayerPrefs.GetInt("unlockableSkill");
            if (unlockable < slots.Length-1)
            {
                Unlockable(index+1);
            }
        }
    }

    private void Unlockable(int index)
    {
        PlayerPrefs.SetInt("unlockableSkill", index);
        slots[index].transform.GetChild(1).gameObject.SetActive(true);
    }

    Sprite GetSprite(int index)
    {
        if (index == 0 || index == 2 || index == 4)
            return fireBallSprite;
        else return boostSprite;
    }

    string GetText(int index)
    {
        string text = null;
        float actualInvul= PlayerPrefs.GetFloat("invulSeconds");

        switch (index)
        {
            case 0:
                {
                    text = "Damage x1.5";
                    PlayerPrefs.SetFloat("fireballDamage", 1.5f);
                    break;
                }
            case 1:
                {
                    text = "Invul x1.5s";
                    PlayerPrefs.SetFloat("invulSeconds", actualInvul * 1.5f);
                    break;
                }
            case 2:
                {
                    text = "Damage x2";
                    PlayerPrefs.SetFloat("fireballDamage", 2);
                    break;
                }
            case 3:
                {
                    text = "6 balls/shot";
                    PlayerPrefs.SetInt("maxFireballs", 6);
                    break;
                }
            case 4:
                {
                    text = "Damage x2.5";
                    PlayerPrefs.SetFloat("fireballDamage", 2.5f);
                    break;
                }
            case 5:
                {
                    text = "Invul x2s\n10 balls/shot";
                    PlayerPrefs.SetFloat("invulSeconds", actualInvul * 2f);
                    PlayerPrefs.SetInt("maxFireballs", 10);
                    break;
                }
        }

        return text;
    }
}
